/* $Id: util.h,v 1.7 2002/04/10 08:20:13 aa5779 Exp $ */
#ifndef UTIL_H
#define UTIL_H

#include <stdlib.h>
#include <stdio.h>
#include <setjmp.h>

const char *libutils_version(void);

int versioncmp(const char *ver1, const char *ver2);

void strupr(char *str);
int isnamesym(int ch);
int isplainnamesym(int ch);

void fatal_error(const char *fmt, ...);
void warning(const char *fmt, ...);

void register_error_handler(void *arg);
void recover_error(const char *fmt, ...);
void unwind_error(void);
char *get_error_desc(void);
void *get_eh_data(void);
jmp_buf *get_error_buf(void);

#define CATCH() (setjmp(*get_error_buf()))

// Safe memory allocation functions

void *xmalloc(size_t size);
void *xcalloc(size_t n, size_t size);
void *xrealloc(void *addr, size_t news);
char *xstrdup(const char *str);

void *alloc_chunk(int size);
void free_chunk(void *chunk, int size);
void reset_free_chunks(void);

#define ALLOC(type) (alloc_chunk(sizeof(type)))
#define FREE(p)	(free_chunk((p), sizeof(*(p))))

// IO functions

int set_line_no(int newnl);

typedef struct mapping
{
	void *data;
	int len;
	int hfile;
	int really_mapped;
} mapping;

int xopen(const char *name, int mode, int rights);
FILE *xfopen(const char *name, const char *mode);

#define PATH_SEP (':')

void *find_filename(const char *name, const char *path, const char *defext, 
					void *(*ioproc)(const char *name), void *(*cproc)(const char *name));

char *load_file(const char *name, const char *path, const char *defext);

mapping *map_file(const char *name, const char *path, const char *defext);
int is_map_eof(mapping *map, const char *pos);
void delete_mapping(mapping *map);


// Common parsing functions

void skip_spaces(const char *start, const char **end);
void skip_spaces_nc(const char *start, const char **end);
int parse_single_char(const char *start, const char **end);
int parse_char(const char *start, const char **end);
char *parse_string(const char *start, const char **end);
char *parse_id(const char *start, const char **end);
char *parse_plain_id(const char *start, const char **end);
int parse_num_token(const char *start, const char **end);
void skip_ifs(const char *start, const char **end, int else_flag);

typedef struct table_t
{
	char *key;
	void *data;
} table_t;

void *lookup_table(table_t *table, const char *key);
int lookup_name(char **names, const char *key);

#define NUMOFITEMS(array) (sizeof(array) / sizeof(*(array)))
#define ENDOF(array) ((array) + NUMOFITEMS(array))

#endif // UTIL_H
